﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Text;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 루트 노드
        /// </summary>
        private TreeNode<CircleNode> rootNode = new TreeNode<CircleNode>(new CircleNode("루트"));

        /// <summary>
        /// 선택 노드
        /// </summary>
        private TreeNode<CircleNode> selectedNode;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 이벤트를 설정한다.

            Load                            += Form_Load;
            Resize                          += canvasPictureBox_Resize;
            this.canvasPictureBox.Paint     += canvasPictureBox_Paint;
            this.canvasPictureBox.MouseDown += canvasPictureBox_MouseDown;
            this.canvasPictureBox.MouseMove += canvasPictureBox_MouseMove;
            this.canvasPictureBox.Resize    += canvasPictureBox_Resize;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            TreeNode<CircleNode> node1 = new TreeNode<CircleNode>(new CircleNode("가"));
            TreeNode<CircleNode> node2 = new TreeNode<CircleNode>(new CircleNode("나"));
            TreeNode<CircleNode> node3 = new TreeNode<CircleNode>(new CircleNode("다"));
            TreeNode<CircleNode> node4 = new TreeNode<CircleNode>(new CircleNode("라"));
            TreeNode<CircleNode> node5 = new TreeNode<CircleNode>(new CircleNode("마"));
            TreeNode<CircleNode> node6 = new TreeNode<CircleNode>(new CircleNode("바"));
            TreeNode<CircleNode> node7 = new TreeNode<CircleNode>(new CircleNode("사"));
            TreeNode<CircleNode> node8 = new TreeNode<CircleNode>(new CircleNode("아"));

            rootNode.AddChild(node1);
            rootNode.AddChild(node2);

            node1.AddChild(node3);
            node1.AddChild(node4);
            node2.AddChild(node5);
            node2.AddChild(node6);
            node2.AddChild(node7);
            node5.AddChild(node8);

            ArrangeTree();
        }

        #endregion
        #region 캔버스 픽처 박스 크기 조정시 처리하기 - canvasPictureBox_Resize(sender, e)

        /// <summary>
        /// 캔버스 픽처 박스 크기 조정시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void canvasPictureBox_Resize(object sender, EventArgs e)
        {
            ArrangeTree();
        }

        #endregion
        #region 캔버스 픽처 박스 마우스 DOWN 처리하기 - canvasPictureBox_MouseDown(sender, e)

        /// <summary>
        /// 캔버스 픽처 박스 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void canvasPictureBox_MouseDown(object sender, MouseEventArgs e)
        {
            if(e.Button != MouseButtons.Right)
            {
                return;
            }

            FindNodeUnderMouse(e.Location);

            if(this.selectedNode != null)
            {
                this.deleteNodeMenuItem.Enabled = (this.selectedNode != rootNode);

                this.contextMenuStrip.Show(this, e.Location);
            }
        }

        #endregion
        #region 캔버스 픽처 박스 마우스 이동시 처리하기 - canvasPictureBox_MouseMove(sender, e)

        /// <summary>
        /// 캔버스 픽처 박스 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void canvasPictureBox_MouseMove(object sender, MouseEventArgs e)
        {
            FindNodeUnderMouse(e.Location);

            if(this.selectedNode == null)
            {
                this.statusLabel.Text = string.Empty;
            }
            else
            {
                this.statusLabel.Text = this.selectedNode.Data.Text;
            }
        }

        #endregion
        #region 캔버스 픽처 박스 페인트시 처리하기 - canvasPictureBox_Paint(sender, e)

        /// <summary>
        /// 캔버스 픽처 박스 페인트시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void canvasPictureBox_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode     = SmoothingMode.AntiAlias;
            e.Graphics.TextRenderingHint = TextRenderingHint.AntiAliasGridFit;

            this.rootNode.DrawTree(e.Graphics);
        }

        #endregion
        
        //////////////////////////////////////////////////////////////////////////////// Function

        #region 트리 배열하기 - ArrangeTree()

        /// <summary>
        /// 트리 배열하기
        /// </summary>
        private void ArrangeTree()
        {
            using(Graphics graphics = canvasPictureBox.CreateGraphics())
            {
                float minimumX = 0;
                float minimumY = 0;

                this.rootNode.Arrange(graphics, ref minimumX, ref minimumY);

                minimumX = (ClientSize.Width - minimumX) / 2;
                minimumY = 10;

                this.rootNode.Arrange(graphics, ref minimumX, ref minimumY);
            }

            this.canvasPictureBox.Refresh();
        }

        #endregion
        #region 마우스 아래 노드 찾기 - FindNodeUnderMouse(point)

        /// <summary>
        /// 마우스 아래 노드 찾기
        /// </summary>
        /// <param name="point">포인트</param>
        private void FindNodeUnderMouse(PointF point)
        {
            using(Graphics graphics = this.canvasPictureBox.CreateGraphics())
            {
                this.selectedNode = this.rootNode.GetNode(graphics, point);
            }
        }

        #endregion
        #region 자식 노드 추가 메뉴 항목 클릭시 처리하기 - addChildNodeMenuItem_Click(sender, e)

        /// <summary>
        /// 자식 노드 추가 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void addChildNodeMenuItem_Click(object sender, EventArgs e)
        {
            AddChildNodeForm form = new AddChildNodeForm();

            if(form.ShowDialog() == DialogResult.OK)
            {
                TreeNode<CircleNode> childNode = new TreeNode<CircleNode>(new CircleNode(form.NodeText));

                this.selectedNode.AddChild(childNode);

                ArrangeTree();
            }
        }

        #endregion
        #region 노드 삭제하기 메뉴 항목 클릭시 처리하기 - deleteNodeMenuItem_Click(sender, e)

        /// <summary>
        /// 노드 삭제하기 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void deleteNodeMenuItem_Click(object sender, EventArgs e)
        {
            if(MessageBox.Show("이 노드를 삭제하시겠습니까?", "확인", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                this.rootNode.DeleteNode(this.selectedNode);

                ArrangeTree();
            }
        }

        #endregion
    }
}